-- Character Utilities Returns
-- Version *TWO*
-- Original script from Kart by freeman#7626, port / remake by FrivolousAqua
-- Assistance from Bloops / JugadorXEI
-- Feel free to modify it and add what you like! I'd be happy to include useful inclusions

-- funny sound
freeslot("sfx_aqboom")
-- Sound arrays to make the code a *bit* more organized
local sounds = {"attack1", "attack2", "boost1", "boost2", "gloat", "hitconfirm",
				"hurt1", "hurt2", "lose", "slow", "talk", "win"}
local playback = {sfx_kattk1, sfx_kattk2, sfx_kbost1, sfx_kbost2, sfx_kgloat,
				  sfx_khitem, sfx_khurt1, sfx_khurt2, sfx_klose, sfx_kslow,
				  sfx_ktalk, sfx_kwin}

-- Display a message showing how to access the commands related to CUR.
-- Should pop up every time a stage is entered.
addHook("MapLoad", function(p)
	print("\133Character Utilities Returns \128has been loaded! Use \34help cur_\34 for a list of commands.")
end)

-- Function for spawning the goalpost wherever it is called.
-- This is pretty much ported from the source code.
-- This was entirely translated to lua by JugadorXEI. (Thank you so much, genuinely.)
local function P_SetupSignObject(sign, pmo, signType)
    local cur = sign
    local prev = nil

    -- Setup the sign itself
    sign.scale = 5 * sign.scale / 2 -- 2.5x
    sign.destscale = sign.scale
    sign.target = pmo
    sign.state = S_SIGN_POLE

    sign.movefactor = sign.z
    sign.z = $ + (576*sign.scale) * P_MobjFlip(sign)
    sign.old_z = sign.z -- interp hijack
    sign.movecount = 1
    sign.extravalue1 = AngleFixed(sign.angle) >> FRACBITS

    -- Setup the overlay pieces
    -- Front
    cur.hnext = P_SpawnMobjFromMobj(sign, 0, 0, 0, MT_SIGN_PIECE)
    cur.hnext.target = sign
    cur.hnext.state = S_SIGN_FACE
    cur.hnext.extravalue1 = 6
    cur.hnext.extravalue2 = 0

    prev = cur
    cur = cur.hnext
    cur.hprev = prev

    -- Player icon
    cur.hnext = P_SpawnMobjFromMobj(sign, 0, 0, 0, MT_SIGN_PIECE)
    cur.hnext.target = sign
    cur.hnext.skin = pmo.skin
    if signType == 4 then
        cur.hnext.state = S_SIGN_ERROR
    else 
        cur.hnext.state = S_KART_SIGN
    end
    
    cur.hnext.extravalue1 = 7
    cur.hnext.extravalue2 = 0

    prev = cur
    cur = cur.hnext
    cur.hprev = prev

    -- Back
    cur.hnext = P_SpawnMobjFromMobj(sign, 0, 0, 0, MT_SIGN_PIECE)
    cur.hnext.target = sign
    cur.hnext.state = S_SIGN_BACK
    cur.hnext.extravalue1 = 6
    cur.hnext.extravalue2 = 2

    prev = cur
    cur = cur.hnext
    cur.hprev = prev

    -- Sides
    cur.hnext = P_SpawnMobjFromMobj(sign, 0, 0, 0, MT_SIGN_PIECE)
    cur.hnext.target = sign
    cur.hnext.state = S_SIGN_SIDE
    cur.hnext.extravalue1 = 30
    cur.hnext.extravalue2 = 1

    prev = cur
    cur = cur.hnext
    cur.hprev = prev

    cur.hnext = P_SpawnMobjFromMobj(sign, 0, 0, 0, MT_SIGN_PIECE)
    cur.hnext.target = sign
    cur.hnext.state = S_SIGN_SIDE
    cur.hnext.extravalue1 = 30
    cur.hnext.extravalue2 = 3

    prev = cur
    cur = cur.hnext
    cur.hprev = prev
end

-- Play any player sound that the player supports.
-- Yes, even you, talk.
-- If "all" is given, then all sounds are played back to back.
COM_AddCommand("cur_playsound", function(p, input)
	if p and p.valid and p.mo and p.mo.valid and not p.spectator then
		local soundmatch = 0 -- Flag to indicate whether or not a sound match was found
		for i = 1, 12 do
			if sounds[i] == input then -- Oh hey, it exists!
				S_StartSound(p.mo, playback[i])
				soundmatch = 1
				break;
			end
		end
		if soundmatch == 0 then -- Maybe look at Kartmaker
			S_StartSound(p.mo, sfx_adderr)
			CONS_Printf(p, "\133ERROR: \128That sound doesn't exist! Valid sounds include: attack1, attack2, boost1, boost2, gloat, hitconfirm, hurt1, hurt2, lose, slow, talk, win")
		end
	else
		S_StartSound(p.mo, sfx_adderr)
		CONS_Printf(p, "\133ERROR: \128Can't do that right now!")
	end
end, 1)

-- Damage the player in a simple spinout.
COM_AddCommand("cur_spinout", function(p)
	if p and p.valid and p.mo and p.mo.valid and not p.spectator then
		P_DamageMobj(p.mo, nil, nil, 5, DMG_NORMAL)
	else
		S_StartSound(p.mo, sfx_adderr)
		CONS_Printf(p, "\133ERROR: \128Can't do that right now!")
	end
end, 1)

-- Make the player tumble.
COM_AddCommand("cur_tumble", function(p)
	if p and p.valid and p.mo and p.mo.valid and not p.spectator then
		P_DamageMobj(p.mo, nil, nil, nil, DMG_TUMBLE)
	else
		S_StartSound(p.mo, sfx_adderr)
		CONS_Printf(p, "\133ERROR: \128Can't do that right now!")
	end
end, 1)

-- Explode the player on command.
-- That's it. Yeah.
COM_AddCommand("cur_kaboom", function(p)
	if p and p.valid and p.mo and p.mo.valid and not p.spectator then
		P_DamageMobj(p.mo, nil, nil, 9999, DMG_INSTAKILL)
	else
		S_StartSound(p.mo, sfx_adderr)
		CONS_Printf(p, "\133ERROR: \128Can't do that right now!")
	end
end, 1)

-- now make it funny
COM_AddCommand("cur_kaboooooom", function(p)
	if p and p.valid and p.mo and p.mo.valid and not p.spectator then
		S_StartSound(p.mo, sfx_aqboom)
		P_DamageMobj(p.mo, nil, nil, 9999, DMG_INSTAKILL)
	else
		S_StartSound(p.mo, sfx_adderr)
		CONS_Printf(p, "\133ERROR: \128Can't do that right now!")
	end
end, 1)

-- Spawns a goalpost wherever you are.
-- Many thanks to JugadorXEI, seriously I mean it
COM_AddCommand("cur_goalpost", function(p)
	if p and p.valid and p.mo and p.mo.valid and not p.spectator then
		S_StartSound(p.mo, sfx_kwin) -- You won! ...wait no you didnt
		local pMo = p.mo
		local sign = P_SpawnMobj(pMo.x, pMo.y, pMo.floorz, MT_SIGN)
		
		sign.angle = pMo.angle + ANGLE_180
		sign.cvmem = 2
		P_SetupSignObject(sign, pMo, 2)
	else
		S_StartSound(p.mo, sfx_adderr)
		CONS_Printf(p, "\133ERROR: \128Can't do that right now!")
	end
end, 1)